﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using EPiServer.Business.Commerce.Payment.Klarna.KlarnaPartPayment;
using EPiServer.Business.Commerce.Payment.Klarna.Orders;
using EPiServer.Framework.Localization;
using EPiServer.ServiceLocation;
using Klarna.Api;
using EPiServer.Logging;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Plugins.Payment;

namespace EPiServer.Business.Commerce.Payment.Klarna
{
    /// <summary>
    /// Base class for Klarna payment gateway.
    /// </summary>
    public abstract class KlarnaPaymentGatewayBase : AbstractPaymentGateway, ISplitPaymentGateway
    {
        [NonSerialized]
        protected readonly ILogger _logger = LogManager.GetLogger(typeof(KlarnaPaymentGatewayBase));
        
        /// <summary>
        /// Activate invoice when order is complete.
        /// When user clicks complete order in commerce manager the transaction type will be Captured.
        /// </summary>
        /// <param name="klarnaClient">The Klarna client api</param>
        /// <param name="payment">The Klarna payment</param>
        /// <param name="shipment">The shipment</param>
        /// <param name="message">The message</param>
        /// <returns>True if capture successful, otherwise false</returns>
        protected virtual bool ProcessCapture(Api klarnaClient, KlarnaPayment payment, Shipment shipment, ref string message)
        {
            try
            {
                var klarnaTransaction = payment.ProviderTransactionID;

                foreach (var articleNo in KlarnaHelper.ArticleNoCollection(payment, shipment))
                {
                    klarnaClient.AddArticleNumber(articleNo.Quantity, articleNo.Number);
                }

                if (shipment != null && shipment.ShippingSubTotal > 0)
                {
                    var index = payment.Parent.Shipments.IndexOf(shipment);
                    klarnaClient.AddArticleNumber(1, string.Format(KlarnaHelper.ShippingFeeCodeTemplate, index + 1));
                }

                int pClassId;
                ActivatePartResponse result;
                if (payment.PclassId != null && int.TryParse(payment.PclassId, out pClassId))
                {
                    result = klarnaClient.ActivatePart(klarnaTransaction, pClassId, false);
                }
                else
                {
                    result = klarnaClient.ActivatePart(klarnaTransaction);
                }
                

                // Set new involve number for first payment to use for next partial capture
                if (!string.IsNullOrEmpty(result.InvoiceNumber))
                {
                    payment.Parent.Payments[0].ProviderTransactionID = result.InvoiceNumber;
                }
            }
            catch (KlarnaException e)
            {
                _logger.Error("Errror when completing order", e);
                message = string.Format("Error code: {0}, message: {1}", e.ErrorCode, e.Message);
                return false;
            }

            return true;
        }

        /// <summary>
        /// Process refund payment.
        /// When user clickd complete return order in commerce manager the transaction type will be Credit.
        /// </summary>
        /// <param name="klarnaClient">The Klarna client api</param>
        /// <param name="payment">The Klarna payment</param>
        /// <param name="shipment">The shipment</param>
        /// <param name="message">The message</param>
        /// <returns>True if refund successful, otherwise false</returns>
        protected virtual bool ProcessRefund(Api klarnaClient, KlarnaPayment payment, Shipment shipment, ref string message)
        {
            try
            {
                var klarnaTransaction = payment.ProviderTransactionID;

                foreach (var lineItem in Shipment.GetShipmentLineItems(shipment))
                {
                    klarnaClient.AddArticleNumber((int)Shipment.GetLineItemQuantity(shipment, lineItem.LineItemId), lineItem.Code);
                }

                klarnaClient.CreditPart(klarnaTransaction);

                // Refund shipping fee
                if (shipment != null && shipment.ShippingSubTotal > 0)
                {
                    klarnaClient.ReturnAmount(klarnaTransaction, (double)shipment.ShippingSubTotal, 0, GoodsFlags.Shipping | GoodsFlags.IncVAT, "Refund shipping fee");
                }
            }
            catch (KlarnaException e)
            {
                _logger.Error("Errror when processing refund", e);
                message = string.Format("Error code: {0}, message: {1}", e.ErrorCode, e.Message);
                return false;
            }

            return true;
        }

        /// <summary>
        /// Processes the payment.
        /// </summary>
        /// <param name="payment">The payment.</param>
        /// <param name="message">The message.</param>
        /// <returns></returns>
        public override bool ProcessPayment(Mediachase.Commerce.Orders.Payment orderPayment, ref string message)
        {
            return ProcessPayment(orderPayment, null, ref message);
        }

        /// <summary>
        /// Processes order payment.
        /// </summary>
        /// <param name="orderPayment">The payment</param>
        /// <param name="shipment">The shipment</param>
        /// <param name="message">The message</param>
        /// <returns></returns>
        public bool ProcessPayment(Mediachase.Commerce.Orders.Payment orderPayment, Shipment shipment, ref string message)
        {
            KlarnaPayment payment = orderPayment as KlarnaPayment;
            // create a Klarna client
            var klarnaClient = InitKlarnaClient();

            #region Capture processing
            if (payment.TransactionType.Equals(TransactionType.Capture.ToString(), StringComparison.OrdinalIgnoreCase))
            {
                return ProcessCapture(klarnaClient, payment, shipment, ref message);
            }
            #endregion Capture processing

            #region Refund processing

            if (payment.TransactionType.Equals(TransactionType.Credit.ToString(), StringComparison.OrdinalIgnoreCase))
            {
                return ProcessRefund(klarnaClient, payment, shipment, ref message);
            }
            #endregion Refund processing

            // When "Complete" or "Refund" shipment in Commerce Manager, this method will be run again with the TransactionType is Capture/Credit
            // Klarna will always return true to bypass the payment process again.
            if (!payment.TransactionType.Equals(TransactionType.Authorization.ToString(), StringComparison.OrdinalIgnoreCase))
            {
                return true;
            }

            return ProcessTransaction(klarnaClient, payment, ref message);
        }

        /// <summary>
        /// Process transaction.
        /// </summary>
        /// <param name="klarnaClient">The Klarna client api</param>
        /// <param name="payment">The Klarna payment</param>
        /// <param name="message">The message</param>
        /// <returns>True if authorize successful, otherwise false</returns>
        protected abstract bool ProcessTransaction(Api klarnaClient, KlarnaPayment payment, ref string message);

        /// <summary>
        /// Creates and returns the Klarna client API.
        /// </summary>
        /// <returns>Klarna client API</returns>
        protected abstract Api InitKlarnaClient();       
    }
}
