using System;
using System.Collections.Generic;
using System.Globalization;
using System.ServiceModel.Activation;
using EPiServer.Data.Dynamic;
using EPiServer.Framework.Localization;
using EPiServer.ServiceLocation;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Website.Helpers;
using Klarna.Api;
using EPiCurrency = Mediachase.Commerce.Currency;

namespace EPiServer.Business.Commerce.Payment.Klarna.Services
{
    /// <summary>
    /// Provides services that used in Klarna payment gateway.
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class KlarnaService : IKlarnaService
    {
        private CartHelper _currentCart = new CartHelper(Cart.DefaultName);

        /// <summary>
        /// Gets the country code of the billing address in current cart of user.
        /// </summary>
        
        /// <returns>The country code</returns>
        public string GetBillingCountryCode()
        {
            return GetCountryCodeForLogo(KlarnaHelper.GetCountryCode());
        }

        /// <summary>
        /// Gets the part payment options which has country code and part payment items.
        /// </summary>
        /// <returns>
        /// The Result object which store success flag, country code and PartPaymentItems
        /// </returns>
        public Result GetPartPaymentOptions()
        {
            double totalAmount = (double) _currentCart.OrderForm.Total;
            string storeName = string.Empty;
            EPiCurrency currency = string.Empty;
            string countryCode = KlarnaHelper.GetCountryCode();

            currency = KlarnaHelper.GetCurrencyByCountry(countryCode);
            storeName = KlarnaHelper.GetStoreNameByCountry(countryCode);

            Result result = new Result() { IsSuccess = false, Message = string.Empty, CountryCode = GetCountryCodeForLogo(countryCode) };
            List<PartPaymentItem> items = new List<PartPaymentItem>();
            try
            {
                Calculation calc = new Calculation();
                // Get the store
                var store = GetStore(storeName);

                // Get the pclass values
                var pClasses = store.LoadAll<EPiPClass>();
                if (pClasses != null)
                {
                    foreach (var epipclass in pClasses)
                    {
                        // Ignore duplidate pclass item base on PClassID
                        if (items.Exists(item => item.Id == epipclass.PClassID.ToString(CultureInfo.InvariantCulture)))
                        {
                            continue;
                        }
                        // Ignore pclass if minAmount is greater than total amount in cart.
                        if (epipclass.MinAmountForPClass > totalAmount)
                        {
                            continue;
                        }

                        // Get the monthly cost from Klarna
                        var monthlyCost = calc.CalculateMonthlyCost(totalAmount, epipclass.ToPClass(), Calculation.CalculateOn.CheckoutPage);
                        if (monthlyCost < 0.0)
                        {
                            continue;
                        }

                        // Round up monthly cost
                        monthlyCost = (int)Math.Ceiling(monthlyCost);

                        PartPaymentItem partPaymentItem = new PartPaymentItem(epipclass.Description,
                                                                                    monthlyCost.ToString(),
                                                                                    currency,
                                                                                    LocalizationService.GetString("/Commerce/klarnapartpayment/month"),
                                                                                    epipclass.PClassID.ToString());
                        items.Add(partPaymentItem);
                    }
                }

                result.IsSuccess = true;
                result.Items = items;
            }
            catch (System.Exception ex)
            {
                result.Message = ex.Message;
                result.IsSuccess = false;
            }

            return result;
        }

        /// <summary>
        /// Gets the country code for logo.
        /// </summary>
        /// <remarks>Klarna supports for 6 countries so if billing address is not supported country sweden is default.</remarks>
        /// <param name="countryCode">The country code.</param>
        /// <returns>Country code for Logo</returns>
        private string GetCountryCodeForLogo(string countryCode)
        {
            var supportedCountry = countryCode.Equals(KlarnaHelper.Swe) || countryCode.Equals(KlarnaHelper.Fin) ||
                                 countryCode.Equals(KlarnaHelper.Deu) || countryCode.Equals(KlarnaHelper.Dnk) ||
                                 countryCode.Equals(KlarnaHelper.Nor) || countryCode.Equals(KlarnaHelper.Nld) || countryCode.Equals(KlarnaHelper.Aut);

            return supportedCountry ? countryCode : KlarnaHelper.Swe;
        }

        /// <summary>
        /// Gets the store based on given store name.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <returns>Return a dynamic data store</returns>
        private static DynamicDataStore GetStore(string name)
        {
            return DynamicDataStoreFactory.Instance.GetStore(name) ??
                   DynamicDataStoreFactory.Instance.CreateStore(name, typeof(EPiPClass));
        }

        private static LocalizationService LocalizationService
        {
            get { return ServiceLocator.Current.GetInstance<LocalizationService>(); }
        }
    }
}